module mod_calibration
!  Purpose:
!  Collects calibrated parameters.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  11/26/2010   T. Yamada    Original code
	use nrtype,only:dp
	implicit none

	!****** lifecycle ******
	integer,parameter :: js = 20
	integer,parameter :: jr = 65
	integer,parameter :: jf = 100

	!****** preference ******
	real(dp),parameter :: beta = 0.984
	real(dp),parameter :: gamma = 2.0
	real(dp),parameter :: sigma = 0.52
	real(dp),parameter :: psi = 0.0

	!****** persistent shock ******
	integer,parameter :: nz = 7
	real(dp) :: rho = 0.98
	real(dp) :: mu_z = 0.0
	real(dp) :: std_eps20 = 0.06
	real(dp) :: std_pr = 0.0004

	!****** transitory shock ******
	integer,parameter :: nt = 2
	real(dp) :: sig_tr = 0.10

	!****** fixed effect ******
	integer,parameter :: nf = 2
	real(dp) :: sig_fx = 0.24

	!****** social security ******
	real(dp),parameter :: rep_rate = 0.4

	!****** available hours for work [1980] ******
	real(dp),parameter :: hmax = 16.0*5.5*4.0*12.0

	!****** TFP factor level [1980] ******
	real(dp),parameter :: tfp = 1.0893

	!****** TFP factor growth rate [1980] ******
	real(dp),parameter :: tfp_gr = 1.0134

	!****** capital share [1980] ******
	real(dp),parameter :: theta = 0.3452

	!****** depreciation rate [1980] ******
	real(dp),parameter :: delta = 0.0949

	!****** capital income tax rate [1980] ******
	real(dp),parameter :: tax_cap = 0.4636
	
	!****** G/Y [1980] ******
	real(dp),parameter :: goy = 0.1544

	!****** demographics ******
	integer,parameter :: ts = 1980
	integer,parameter :: tc = 2000
	integer,parameter :: tt = 2055
	integer,parameter :: tf = 2200

end module mod_calibration