module mod_variables
!  Purpose:
!  Collect all global variables.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  11/26/2010   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	implicit none

	!****** exogenous data ******
	integer :: iter = 1
	real(dp),dimension(25:85,4) :: var_cons_1984_1999
	real(dp),dimension(25:85,4) :: var_erng_1984_1999

	!****** demographics ******
	real(dp),dimension(js:jf) :: surv
	real(dp),dimension(js:jf) :: mu
	real(dp) :: frac_ret

	!****** efficiency profiles ******
	real(dp),dimension(js:jr) :: kappa

	!****** persistent shock ******
	real(dp),dimension(nz,js:jr) :: endow
	real(dp),dimension(nz,nz) :: prob

	!****** transitory shocks ******
	real(dp),dimension(nt) :: shock_tr,prob_tr

	!****** fixed effect ******
	real(dp),dimension(nf) :: shock_fx,prob_fx

	!****** current asset grid ******
	integer,parameter :: na = 50
	real(dp),parameter :: amax = 20000.0
	real(dp),parameter :: amin = 0.0
	real(dp),dimension(na) :: grcap1

	!****** stationary distribution ******
	integer,parameter :: nd = 5000
	real(dp),dimension(nd) :: grcap2

	!****** computation time ******
	real(dp) time_begin,time_end

	!****** structured variables ******
	type :: policy_function
		real(dp),dimension(na,nz,nt,nf,js:jr) :: y
		real(dp),dimension(na,nz,nt,nf,js:jr) :: l
		real(dp),dimension(na,nf,jr+1:jf) :: o
	end type policy_function

	type :: policy_egm
		real(dp),dimension(0:na,nz,nt,nf,js:jr) :: cy
		real(dp),dimension(0:na,nz,nt,nf,js:jr) :: gy
		real(dp),dimension(1:na,nz,nt,nf,js:jr) :: wh
		real(dp),dimension(0:na,nf,jr+1:jf) :: co
		real(dp),dimension(0:na,nf,jr+1:jf) :: go
	end type policy_egm

	type :: phi_function
		real(dp),dimension(nd,nz,nt,nf,js:jr) :: y
		real(dp),dimension(nd,nf,jr+1:jf) :: o
	end type phi_function

end module mod_variables