subroutine end_grid_method(rent,wage,tax_ss,tax_lab,H_agg,beq,policy)
!  Purpose:
!  Compute policy functions using the Endogenous Grid Method.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  07/08/2007   T. Yamada    Original code
!  11/08/2008   T. Yamada    Revised to submit JEDC
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_interpolation,only:interp1
	use mod_EulerEq
	implicit none

	!****** input ******
	real(dp),intent(in) :: rent,wage,tax_ss,tax_lab,beq,H_agg

	!****** output ******
	type(policy_function),intent(out) :: policy

	!****** local variables ******
	integer :: i,j,k,l,ia,status
	real(dp) :: rhs,c_tilde,cons,earning,ss
	real(dp),dimension(na) :: a_prime
	!/* consumption function */
	type(policy_egm) :: egm1
	real(dp),allocatable,dimension(:,:,:,:,:) :: asset1
	real(dp),allocatable,dimension(:,:,:) :: asset2
	!-----------------------------

	!/* current financial asset
	allocate(asset1(na,nz,nt,nf,js:jr),STAT=status)
	allocate(asset2(na,nf,jr+1:jf),STAT=status)
	asset1 = 0.0
	asset2 = 0.0

	!/* next period's choice
	a_prime = grcap1

	!/* consumption function over cash on hand */
	do l = 1,nf
		do ia = jf,js,-1
			select case (ia)
			case(jf)
				egm1%co(0,l,ia) = 0.0
				egm1%go(0,l,ia) = 0.0
				ss = rep_rate*wage*H_agg*shock_fx(l)
				egm1%go(1:na,l,ia) = (1.0+(1.0-tax_cap)*rent)*(grcap1+beq) + ss
				egm1%co(1:na,l,ia) = egm1%go(1:na,l,ia)
				asset2(1:na,l,ia)  = grcap1
			case(jr+1:jf-1)
				egm1%co(0,:,ia) = 0.0
				egm1%go(0,:,ia) = 0.0
				do i = 1,na
					!/* marginal value
					rhs = mv_retire(a_prime(i),ia,l,rent,wage,H_agg,beq,egm1%co,egm1%go)
					!/* inverse of marginal value
					egm1%co(i,l,ia) = inverse_mu_zero_labor(rhs)
					egm1%go(i,l,ia) = egm1%co(i,l,ia) + tfp_gr*a_prime(i)
				end do
				ss = rep_rate*wage*H_agg*shock_fx(l)
				!/* retrieve current financial asset
				asset2(:,l,ia) = (egm1%go(1:na,l,ia)-ss)/(1.0+(1.0-tax_cap)*rent)-beq
			case(js:jr)
				egm1%cy(0,:,:,:,ia) = 0.0
				egm1%gy(0,:,:,:,ia) = 0.0
				do k = 1,nt
					do j = 1,nz
						do i = 1,na
							!/* marginal value
							if (ia == jr) then
								rhs = mv_retire(a_prime(i),ia,l,rent,wage,H_agg,beq,egm1%co,egm1%go)
							else
								rhs = mv_worker(a_prime(i),ia,j,l,rent,wage,tax_ss,tax_lab,beq,egm1%cy,egm1%gy)
							end if
							!/* inverse of marginal value
							egm1%cy(i,j,k,l,ia) = inverse_mu_labor(rhs,ia,j,k,l,wage,tax_ss,tax_lab)
							egm1%wh(i,j,k,l,ia) = labor_from_cons(egm1%cy(i,j,k,l,ia),wage,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss,tax_lab)
							if (egm1%wh(i,j,k,l,ia) == 0.0) then
								egm1%cy(i,j,k,l,ia) = inverse_mu_zero_labor(rhs)
							end if
							c_tilde = egm1%cy(i,j,k,l,ia) + (1.0-tax_ss-tax_lab)*wage*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*(hmax-egm1%wh(i,j,k,l,ia))
							egm1%gy(i,j,k,l,ia) = c_tilde + tfp_gr*a_prime(i)
						end do
						earning = (1.0-tax_ss-tax_lab)*wage*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hmax
						!/* retrieve current financial asset
						asset1(:,j,k,l,ia) = (egm1%gy(1:na,j,k,l,ia)-earning)/(1.0+(1.0-tax_cap)*rent)-beq
					end do
				end do
			end select
		end do
	end do

	!/* policy function over asset */
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						!/* next asset
						do i = 1,na
							if (asset1(1,j,k,l,ia) > grcap1(i)) then
								policy%y(i,j,k,l,ia) = amin
							else
								policy%y(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),a_prime,grcap1(i))
							end if
							if (policy%y(i,j,k,l,ia) < amin) then
								policy%y(i,j,k,l,ia) = amin
							end if
						end do
						!/* hours worked
						do i = 1,na
							if (asset1(1,j,k,l,ia) > grcap1(i)) then
								cons = interp1(asset1(:,j,k,l,ia),egm1%cy(1:na,j,k,l,ia),grcap1(i))
								policy%l(i,j,k,l,ia) = labor_from_cons(cons,wage,kappa(ia),endow(j,ia),shock_tr(k),shock_fx(l),tax_ss,tax_lab)
							else
								policy%l(i,j,k,l,ia) = interp1(asset1(:,j,k,l,ia),egm1%wh(:,j,k,l,ia),grcap1(i))
							end if
							if (policy%l(i,j,k,l,ia) > hmax) then
								policy%l(i,j,k,l,ia) = hmax
							elseif (policy%l(i,j,k,l,ia) < 0.0) then
								policy%l(i,j,k,l,ia) = 0.0
							end if
						end do
					end do
				end do
			end do
		case(jr+1:jf-1)
			do l = 1,nf
				do i = 1,na
					if (asset2(1,l,ia) > grcap1(i)) then
						policy%o(i,l,ia) = amin
					else
						policy%o(i,l,ia) = interp1(asset2(:,l,ia),a_prime,grcap1(i))
					end if
					if (policy%o(i,l,ia) < amin) then
						policy%o(i,l,ia) = amin
					end if
				end do
			end do
		case(jf)
			policy%o(:,:,ia) = 0.0
		end select
	end do

	deallocate(asset1)
	deallocate(asset2)

end subroutine end_grid_method