subroutine law_of_motion(policy,phi,K_mean_age,K_next_age,H_mean_age,H_hour_age)
!  Purpose:
!  Given a policy function, compute a wealth distribution for each age.
!  This subroutine also compute aggregate capital and labor for each age.
!  
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  07/09/2007   T. Yamada    Original
!  11/08/2008   T. Yamada    Revised to Submit JEDC
!  10/20/2010   T. Yamada    Modified in Initial Distribution
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_interpolation,only:locate
	use mod_generate_grid
	implicit none

	!****** input *******
	type(policy_function),intent(in) :: policy

	!****** output *******
	type(phi_function),intent(out) :: phi
	real(dp),intent(out),dimension(js:jf) :: K_mean_age,K_next_age
	real(dp),intent(out),dimension(js:jr) :: H_mean_age,H_hour_age

	!****** local variables ******
	integer :: i,j,k,l,ia,id,iz,izz,it,itt,ip,status
	integer :: aloc,z_0
	real(dp) :: bunbo,bunsi,wta,frac_l,frac_h
	real(dp),allocatable,dimension(:,:,:,:,:) :: saving1
	real(dp),allocatable,dimension(:,:,:,:,:) :: hours
	integer,allocatable,dimension(:,:,:,:,:) :: loca1
	real(dp),allocatable,dimension(:,:,:,:,:) :: wght1
	real(dp),allocatable,dimension(:,:,:) :: saving2
	integer,allocatable,dimension(:,:,:) :: loca2
	real(dp),allocatable,dimension(:,:,:) :: wght2
	!-----------------------------

	!/* policy function over finer grid */
	allocate(saving1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(saving2(nd,nf,jr+1:jf),STAT=status)
	allocate(hours(nd,nz,nt,nf,js:jr),STAT=status)
	saving1 = 0.0
	saving2 = 0.0
	hours = 0.0
	call finer_policy(policy,saving1,hours,saving2)

	!*** set-up for distribution iteration ***
	allocate(wght1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(loca1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(wght2(nd,nf,jr+1:jf),STAT=status)
	allocate(loca2(nd,nf,jr+1:jf),STAT=status)
	wght1 = 0.0
	wght2 = 0.0
	loca1 = 0
	loca2 = 0
	do l = 1,nf
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							aloc = locate(grcap2,saving1(i,j,k,l,ia))
							if (aloc >= nd) then
								aloc = nd - 1
							elseif (aloc <= 1) then
								aloc = 1
							end if
							loca1(i,j,k,l,ia) = aloc
							bunsi = saving1(i,j,k,l,ia) - grcap2(aloc)
							bunbo = grcap2(aloc+1) - grcap2(aloc)
							wght1(i,j,k,l,ia) = bunsi / bunbo
							if (wght1(i,j,k,l,ia) > 1.0) then
								wght1(i,j,k,l,ia) = 1.0
							elseif (bunsi < 0.0) then
								wght1(i,j,k,l,ia) = 0.0
							end if
						end do
					end do
				end do
			case(jr+1:jf)
				do i = 1,nd
					aloc = locate(grcap2,saving2(i,l,ia))
					if (aloc >= nd) then
						aloc = nd - 1
					elseif (aloc <= 1) then
						aloc = 1
					end if
					loca2(i,l,ia) = aloc
					bunsi = saving2(i,l,ia) - grcap2(aloc)
					bunbo = grcap2(aloc+1) - grcap2(aloc)
					wght2(i,l,ia) = bunsi / bunbo
					if (wght2(i,l,ia) > 1.0) then
						wght2(i,l,ia) = 1.0
					elseif (bunsi < 0.0) then
						wght2(i,l,ia) = 0.0
					end if
				end do
			end select
		end do
	end do

	!/* initial distribution: E[z_20]=1 */
	phi%y = 0.0
	phi%o = 0.0
	z_0 = locate(endow(:,js),1.0d0)
	frac_l = (endow(z_0+1,js)-1.0)/(endow(z_0+1,js)-endow(z_0,js))
	frac_h = 1.0-frac_l
	do l = 1,nf
		do k = 1,nt
			phi%y(1,z_0  ,k,l,js) = prob_fx(l)*frac_l*prob_tr(k)
			phi%y(1,z_0+1,k,l,js) = prob_fx(l)*frac_h*prob_tr(k)
		end do
	end do

	!/* iterate distribution forwardly */
	do ip = 1,nf
		do ia = js,jf-1
			select case(ia)
			case(js:jr-1)
				do iz = 1,nz
					do it = 1,nt
						do id = 1,nd
							do itt = 1,nt
								do izz = 1,nz
									aloc = loca1(id,iz,it,ip,ia)
									wta  = wght1(id,iz,it,ip,ia)
									phi%y(aloc  ,izz,itt,ip,ia+1) = (1.0-wta)*prob(iz,izz)*phi%y(id,iz,it,ip,ia)*prob_tr(itt) + phi%y(aloc,  izz,itt,ip,ia+1)
									phi%y(aloc+1,izz,itt,ip,ia+1) =      wta *prob(iz,izz)*phi%y(id,iz,it,ip,ia)*prob_tr(itt) + phi%y(aloc+1,izz,itt,ip,ia+1)
								end do
							end do
						end do
					end do
				end do
			case(jr)
				do iz = 1,nz
					do it = 1,nt
						do id = 1,nd
							aloc = loca1(id,iz,it,ip,ia)
							wta  = wght1(id,iz,it,ip,ia)
							phi%o(aloc,  ip,ia+1) = (1.0-wta)*phi%y(id,iz,it,ip,ia) + phi%o(aloc,  ip,ia+1)
							phi%o(aloc+1,ip,ia+1) =      wta *phi%y(id,iz,it,ip,ia) + phi%o(aloc+1,ip,ia+1)
						end do
					end do
				end do
			case(jr+1:jf)
				do id = 1,nd
					aloc = loca2(id,ip,ia)
					wta  = wght2(id,ip,ia)
					phi%o(aloc  ,ip,ia+1) = (1.0-wta)*phi%o(id,ip,ia) + phi%o(aloc,  ip,ia+1)
					phi%o(aloc+1,ip,ia+1) =      wta *phi%o(id,ip,ia) + phi%o(aloc+1,ip,ia+1)
				end do
			end select
		end do
	end do
	deallocate(wght1,STAT=status)
	deallocate(loca1,STAT=status)
	deallocate(wght2,STAT=status)
	deallocate(loca2,STAT=status)

	!/* aggregate capital */
	K_next_age = 0.0
	K_mean_age = 0.0
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							K_mean_age(ia) = phi%y(i,j,k,l,ia)*grcap2(i) + K_mean_age(ia)
							K_next_age(ia) = phi%y(i,j,k,l,ia)*saving1(i,j,k,l,ia) + K_next_age(ia)
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					K_mean_age(ia) = phi%o(i,l,ia)*grcap2(i) + K_mean_age(ia)
					K_next_age(ia) = phi%o(i,l,ia)*saving2(i,l,ia) + K_next_age(ia)
				end do
			end do
		end select
	end do

	!/* labor supply and hours worked */
	H_mean_age = 0.0
	H_hour_age = 0.0
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						!/* labor supply in efficiency unit
						H_mean_age(ia) = phi%y(i,j,k,l,ia)*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hours(i,j,k,l,ia) + H_mean_age(ia)
						!/* hours worked
						H_hour_age(ia) = phi%y(i,j,k,l,ia)*hours(i,j,k,l,ia) + H_hour_age(ia)
					end do
				end do
			end do
		end do
	end do

	!/* deallocate reserved variables
	deallocate(saving1,STAT=status)
	deallocate(saving2,STAT=status)
	deallocate(hours,STAT=status)

end subroutine law_of_motion


subroutine finer_policy(policy,saving1,hours,saving2)
!  Purpose:
!  New policy function over finer grids.
! 
!  Record of revisions:
!    Date       Programmer   Description of change
!  ==========   ==========   =====================
!  11/04/2006   T. Yamada    Original
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_interpolation,only:interp1
	implicit none
	!****** input *******
	type(policy_function),intent(in) :: policy
	!****** output ******
	real(dp),intent(out),dimension(nd,nz,nt,nf,js:jr) :: saving1,hours
	real(dp),intent(out),dimension(nd,nf,jr+1:jf) :: saving2
	!****** local variables ******
	integer :: i,j,k,l,ia,status
	!-----------------------------
	!/* re-compute policy function for finer grids
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							saving1(i,j,k,l,ia) = interp1(grcap1,policy%y(:,j,k,l,ia),grcap2(i))
							hours(i,j,k,l,ia)   = interp1(grcap1,policy%l(:,j,k,l,ia),grcap2(i))
							if (saving1(i,j,k,l,ia) < amin) then
								saving1(i,j,k,l,ia) = amin
							end if
							if (hours(i,j,k,l,ia) > hmax) then
								hours(i,j,k,l,ia) = hmax
							elseif (hours(i,j,k,l,ia) < 0.0) then
								hours(i,j,k,l,ia) = 0.0
							end if
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					saving2(i,l,ia) = interp1(grcap1,policy%o(:,l,ia),grcap2(i))
					if (saving2(i,l,ia) < amin) then
						saving2(i,l,ia) = amin
					end if
				end do
			end do
		end select
	end do
end subroutine finer_policy