subroutine outer_loop
!  Purpose:
!  Outer loop for estimation.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  11/26/2010   T. Yamada    Original
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use UMPOL_INT
	implicit none
	external middle_loop

	!****** local variables ******
	integer :: i1,i2,i3,i4,i5
	integer,parameter :: dim=5
	real(dp) :: loglh,inc1,inc2,inc3,inc4,inc5
	real(dp),dimension(dim) :: init,est
	integer,dimension(dim) :: min_loc

	!****** #grid for grid search ******
!	integer,parameter :: g1 = 3
!	integer,parameter :: g2 = 5
!	integer,parameter :: g3 = 6
!	integer,parameter :: g4 = 5
!	integer,parameter :: g5 = 4
	integer,parameter :: g1 = 2
	integer,parameter :: g2 = 5
	integer,parameter :: g3 = 3
	integer,parameter :: g4 = 5
	integer,parameter :: g5 = 3

	!****** grid search ranges ******
!	real(dp),parameter :: beta_l = 0.96
!	real(dp),parameter :: beta_h = 0.98
!	real(dp),parameter :: std_eps20_l = 0.04
!	real(dp),parameter :: std_eps20_h = 0.08
!	real(dp),parameter :: std_pr_l = 0.0
!	real(dp),parameter :: std_pr_h = 0.0005
!	real(dp),parameter :: sig_tr_l = 0.04
!	real(dp),parameter :: sig_tr_h = 0.08
!	real(dp),parameter :: sig_fx_l = 0.22
!	real(dp),parameter :: sig_fx_h = 0.25
	real(dp),parameter :: beta_l = 0.97
	real(dp),parameter :: beta_h = 0.98
	real(dp),parameter :: std_eps20_l = 0.04
	real(dp),parameter :: std_eps20_h = 0.08
	real(dp),parameter :: std_pr_l = 0.0004
	real(dp),parameter :: std_pr_h = 0.0006
	real(dp),parameter :: sig_tr_l = 0.04
	real(dp),parameter :: sig_tr_h = 0.08
	real(dp),parameter :: sig_fx_l = 0.23
	real(dp),parameter :: sig_fx_h = 0.25

	!****** residuals ******
	real(dp),dimension(g1) :: vec1
	real(dp),dimension(g2) :: vec2
	real(dp),dimension(g3) :: vec3
	real(dp),dimension(g4) :: vec4
	real(dp),dimension(g5) :: vec5
	real(dp),dimension(g1,g2,g3,g4,g5) :: resid

	!****** Nelder-Mead subroutine parameters ******
	real(dp) :: lens   = 0.0001
	real(dp) :: ftol   = 1.0e-004
	integer :: maxfcn = 1000
	!-----------------------------------------------

	!/* increment */
	inc1 = (beta_h - beta_l)/real(g1-1)
	inc2 = (std_eps20_h - std_eps20_l)/real(g2-1)
	inc3 = (std_pr_h - std_pr_l)/real(g3-1)
	inc4 = (sig_tr_h - sig_tr_l)/real(g4-1)
	inc5 = (sig_fx_h - sig_fx_l)/real(g5-1)

	!/* grid search */
	open(40,file='grid_search.txt')
	open(39,file='log_likelihood.txt')
	do i1 = 1,g1
		vec1(i1) = beta_l + (real(i1)-1)*inc1
		do i2 = 1,g2
			vec2(i2) = std_eps20_l + (real(i2)-1)*inc2
			do i3 = 1,g3
				vec3(i3) = std_pr_l + (real(i3)-1)*inc3
				do i4 = 1,g4
					vec4(i4) = sig_tr_l + (real(i4-1))*inc4
					do i5 = 1,g5
						vec5(i5) = sig_fx_l + (real(i5-1))*inc5
						est(1) = vec1(i1)
						est(2) = vec2(i2)
						est(3) = vec3(i3)
						est(4) = vec4(i4)
						est(5) = vec5(i5)
						write(*,*) "Iteration Counter:", i1, i2, i3, i4, i5
						call middle_loop(dim,est,resid(i1,i2,i3,i4,i5))
						write(*,*) "Log-likelihood   :", resid(i1,i2,i3,i4,i5)
						write(*,*) ""
						write(39,*) resid(i1,i2,i3,i4,i5)
						write(40,*) vec1(i1)
						write(40,*) vec2(i2)
						write(40,*) vec3(i3)
						write(40,*) vec4(i4)
						write(40,*) vec5(i5)
						write(40,*) resid(i1,i2,i3,i4,i5)
						write(40,*) ""
					end do
				end do
			end do
		end do
	end do
	close(40)
	close(39)

	!/* search a minimum */
	min_loc = minloc(resid)
	est(1) = vec1(min_loc(1))
	est(2) = vec2(min_loc(2))
	est(3) = vec3(min_loc(3))
	est(4) = vec4(min_loc(4))
	est(5) = vec5(min_loc(5))

	!/* grid search result */
	init = est
	print*, "-+-+-+- Grid Search Results -+-+-+-"
	print*, "rho      : ", est(1)
	print*, "sigma_eta: ", est(2)
	print*, "slope    : ", est(3)
	print*, "sigma_tr : ", est(4)
	print*, "sigma_fx : ", est(5)
	print*, ""

	!/* output estimated results
	open(41,file='grid_serach_result.txt')
		write(41,*) est(1)
		write(41,*) est(2)
		write(41,*) est(3)
		write(41,*) est(4)
		write(41,*) est(5)
		write(41,*) resid(min_loc(1),min_loc(2),min_loc(3),min_loc(4),min_loc(5))
	close(41)

	!/* use the Nelder-Mead simplex algorithm to find a minimum */
!	write(*,*) "-+-+-+- Find a minimum using IMSL solver -+-+-+-"
!	init(1) = 0.97
!	init(2) = 0.06
!	init(3) = 0.0005
!	init(4) = 0.06
!	init(5) = 0.24
!	call UMPOL(middle_loop,est,XGUESS=init,S=lens,FTOL=ftol,MAXFCN=maxfcn,FVALUE=loglh)

	!/* output estimated results
!	open(42,file='estimated_result.txt')
!		write(42,*) est(1)
!		write(42,*) est(2)
!		write(42,*) est(3)
!		write(42,*) est(4)
!		write(42,*) est(5)
!		write(42,*) loglh
!	close(42)

end subroutine outer_loop