program main
!  Purpose:
!  Main program file to compute a miss-predicted path:
!  "Income Risk, Consumption Inequality and Macroeconomy."
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  10/20/2006   T. Yamada    Original code
!  02/24/2007   T. Yamada    Modified to include TFP change
!  01/25/2009   T. Yamada    Revised to submit JEDC
!  02/03/2011   T. Yamada    Reviesd to resubmit JEDC
	include 'link_fnl_shared.h'
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_generate_grid
	use mod_preamble
	implicit none

	call cpu_time(time_begin)
	write(*,*) "Start to Compute a Transitional Path of a Life Cycle Model"
	write(*,*) ""

	!/* survival probability & demographics */
	call demographics

	!/* idiosyncratic wage risk */
	call wage_risk

	!/* grid for financial asset */
	call grid_double_exp(amin,amax,na,grcap1)
	call grid_double_exp(amin,amax,nd,grcap2)

	!/* set exogenous path */
	call set_exogenous_path

	!/* initial steady state /*
	call steady_state_ini

	!/* final steady state */
	call steady_state_fin

	!/* set initial paths */
	call set_initial_path

	!/* transition path */
	call transition

	!/* computation time */
	call cpu_time(time_end)
	write(*,*)  "Program Finished Sucessfully in", time_end-time_begin, "seconds"

end program main