module mod_variables
!  Purpose:
!  Collect all global variables.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  10/12/2006   T. Yamada    Original code
!  01/25/2009   T. Yamada    Revised to submit JEDC
!  02/03/2011   T. Yamada    Reviesd to resubmit JEDC
	use nrtype,only:dp
	use mod_calibration
	implicit none

	!****** demographics ******
	real(dp),dimension(js:jf,ts:tf) :: surv
	real(dp),dimension(js:jf,ts:tf) :: mu
	real(dp),dimension(ts:tf) :: frac_ret

	!****** age-efficiency profiles ******
	real(dp),dimension(js:jr) :: kappa

	!****** persistent shock ******
	real(dp),dimension(nz,js:jr) :: endow
	real(dp),dimension(nz,nz) :: prob

	!****** transitory shock ******
	real(dp),dimension(nt) :: shock_tr,prob_tr

	!****** fixed effect ******
	real(dp),dimension(nf) :: shock_fx,prob_fx

	!****** capital grid ******
	integer,parameter :: na = 50
	real(dp),parameter :: amax = 20000.0
	real(dp),parameter :: amin = 0.0
	real(dp),dimension(na) :: grcap1

	!****** stationary distribution ******
	integer,parameter :: nd = 5000
	real(dp),dimension(nd) :: grcap2

	!****** computation time ******
	real(dp) time_begin,time_end

	!****** structured variables ******
	type :: state_space
		real(dp) :: tfp_gr
		real(dp) :: hmax
		real(dp) :: theta
		real(dp) :: delta
		real(dp) :: tax_cap
		real(dp) :: goy
		real(dp) :: gov_exp
		real(dp) :: cap
		real(dp) :: lab
		real(dp) :: rent
		real(dp) :: wage
		real(dp) :: beq
		real(dp) :: tax_ss
		real(dp) :: tax_lab
	end type state_space


	type :: predicted_path
		real(dp) :: tfp_gr
		real(dp) :: hmax
		real(dp) :: lab
		real(dp) :: rent
		real(dp) :: wage
		real(dp) :: beq
		real(dp) :: tax_lab
		real(dp) :: tax_cap
		real(dp) :: tax_ss
	end type predicted_path


	type :: policy_function
		real(dp),dimension(na,nz,nt,nf,js:jr) :: y
		real(dp),dimension(na,nz,nt,nf,js:jr) :: l
		real(dp),dimension(na,nf,jr+1:jf) :: o
	end type policy_function

	type :: policy_egm
		real(dp),dimension(0:na,nz,nt,nf,js:jr) :: cy
		real(dp),dimension(0:na,nz,nt,nf,js:jr) :: gy
		real(dp),dimension(1:na,nz,nt,nf,js:jr) :: wh
		real(dp),dimension(0:na,nf,jr+1:jf) :: co
		real(dp),dimension(0:na,nf,jr+1:jf) :: go
	end type policy_egm

	type :: phi_function
		real(dp),dimension(nd,nz,nt,nf,js:jr) :: y
		real(dp),dimension(nd,nf,jr+1:jf) :: o
	end type

	type(state_space) :: state_ini
	type(state_space) :: state_fin
	type(state_space) :: state0(ts:tf)
	type(state_space) :: state1(ts:tf)
	type(predicted_path) :: predict(ts:tf)
	type(phi_function) :: phi_ini
	type(policy_egm) :: egm_fin
	type(policy_function) :: policy
	type(policy_function) :: policy_pre(ts:tf)

end module mod_variables