subroutine demographics
!  Purpose:
!  Load survival probabilities and calculate demographic structure.
!
!  Data source:
!  the National Institute of Population and Social Security Research
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  01/26/2007   T. Yamada    Original code
!  11/20/2007   T. Yamada    Updated to h18 version
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none

	!****** local variables ******
	integer :: i,j,status
	real(dp),allocatable,dimension(:) :: surv_all
	real(dp),allocatable,dimension(:,:) :: surv_1980_2055,mu_tran
	real(dp),dimension(ts:tt-1) :: pop_growth_1980_2054
	real(dp),dimension(tt:2065) :: pop_growth_2055_2065
	real(dp),dimension(0:jf) :: mu_1980,nu
	real(dp),dimension(0:jf,0:jf) :: GAM
	!-----------------------------

	allocate(surv_all(jf*(tt-ts+1)),STAT=status)
	allocate(surv_1980_2055(0:jf,ts:tt),STAT=status)
	allocate(mu_tran(0:jf,ts:tf),STAT=status)
	surv_all       = 0.0
	surv_1980_2055 = 0.0
	mu_tran        = 0.0

	!/* survival probability from 1980 to 2055
	open(10,file='surv_1980_2055.in')
		read(10,*) surv_all
	close(10)
	do i = ts,tt
		surv_1980_2055(0:jf-1,i) = surv_all((i-ts)*jf+1:jf*(i-ts+1))
	end do
	surv_1980_2055(jf,:) = 0.0

	!/* use actual age distribution
	open(11,file='pop_1980.in')
		read(11,*) mu_1980
	close(11)
	mu_tran(:,ts) = mu_1980

	!/* fertility rate from 1980 to 2054: medium variant
	open(12,file='pop_growth_1980_2054_medium.in')
		read(12,*) pop_growth_1980_2054
	close(12)

	!/* linear convergence of population growth rate from 2055 to 2065
	pop_growth_2055_2065(2065) = 1.0
	do i = 2055,2064,1
		pop_growth_2055_2065(i) = pop_growth_1980_2054(2054) + ((pop_growth_2055_2065(2065)-pop_growth_1980_2054(2054))/real(2064-2054))*(i-2054)
	end do

	!/* population dynamics from 1980 to 2200
	do i = ts,tf-1
		GAM = 0.0
		select case(i)
		!/* 1980-2054
		case(1:tt-1)
			do j = 1,jf
				GAM(j,j-1) = surv_1980_2055(j,i)
			end do
			GAM(0,0)       = pop_growth_1980_2054(i)
			mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
		!/* 2055-2065
		case(tt:2065)
			do j = 1,jf
				GAM(j,j-1) = surv_1980_2055(j,tt)
			end do
			GAM(0,0)       = pop_growth_2055_2065(i)
			mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
		!/* 2066-2200
		case(2066:)
			do j = 1,jf
				GAM(j,j-1) = surv_1980_2055(j,tt)
			end do
			GAM(0,0)       = 1.0
			mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
		end select
	end do

	!/* population dynamics
	do i = ts,tf
		mu(:,i) = mu_tran(js:jf,i)/sum(mu_tran(js:jf,i),dim=1)
		frac_ret(i) = sum(mu_tran(jr+1:jf,i),dim=1) / sum(mu_tran(js:jf,i),dim=1)
	end do

	!/* survival probabilities
	surv(:,ts:tt) = surv_1980_2055(js:jf,ts:tt)
	do i = tt+1,tf
		surv(:,i) = surv_1980_2055(js:jf,tt)
	end do

	deallocate(surv_all)
	deallocate(surv_1980_2055)
	deallocate(mu_tran)

end subroutine demographics