program main
!  Purpose:
!  Main program file to compute a steady state:
!  "Income Risk, Consumption Inequality, and Macroeconomy."
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  10/20/2006   T. Yamada    Original code
!  01/25/2009   T. Yamada    Revised to submit JEDC
!  12/02/2010   T. Yamada    Reviesd to resubmit JEDC
	include 'link_fnl_shared.h'
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_generate_grid
	use mod_preamble
	implicit none

	call cpu_time(time_begin)
	write(*,*) "Start to Compute a Steady State of a Life Cycle Model"
	write(*,*) ""

	write(*,*) "Exogenous Macro Parameters:"
	write(*,*) "1: Initial Steady State: 1980."
	write(*,*) "2: Final Steady State: 2200."
	write(*,*) "3: Japanese Economy in 1970s."
	write(*,*) "4: Japanese Economy in 1980s."
	write(*,*) "5: Japanese Economy in 1990s."
	write(*,*) "6: Counterfactual Simulation."
	read*, flag_year
	write(*,*) ""

	write(*,*) "Demographics:"
	write(*,*) "1: Use Population Distribution in 1980."
	write(*,*) "2: Use Stationary Population Distribution."
	read*, flag_pop
	write(*,*) ""

	!/* set exogenous variables */
	call set_initial

	!/* survival probability & demographics */
	call demographics

	!/* idiosyncratic wage risk */
	call wage_risk

	!/* grid for financial asset */
	call grid_double_exp(amin,amax,na,grcap1)
	call grid_double_exp(amin,amax,nd,grcap2)

	call steady_state

	!/* computation time */
	call cpu_time(time_end)
	write(*,*) "Program Finished Sucessfully in", time_end-time_begin, "Seconds"

end program main