module mod_calibration
!  Purpose:
!  Collects calibrated parameters.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  10/20/2006   T. Yamada    Original code
!  01/25/2009   T. Yamada    Revised to submit JEDC
!  12/02/2010   T. Yamada    Reviesd to resubmit JEDC
	use nrtype,only:dp
	implicit none

	!****** preference ******
	real(dp),parameter :: beta = 0.984
	real(dp),parameter :: gamma = 2.0
	real(dp),parameter :: sigma = 0.52
	real(dp),parameter :: psi = 0.0
	real(dp),parameter :: hmax80 = 16.0*5.5*4.0*12.0
	real(dp),parameter :: hmax00 = 16.0*5.0*4.0*12.0

	!****** lifecycle ******
	integer,parameter :: js = 20
	integer,parameter :: jr = 65
	integer,parameter :: jf = 100

	!****** persistent shock ******
	integer,parameter :: nz = 7
	real(dp),parameter :: rho = 0.97 !0.98
	real(dp),parameter :: mu_z = 0.0
	real(dp),parameter :: std_eps20 = 0.06
	real(dp),parameter :: std_pr = 0.0005 !0.0004

	!****** transitory shock ******
	integer,parameter :: nt = 2
	real(dp),parameter :: sig_tr = 0.06 !0.10
	real(dp),parameter :: mu_tr = -(sig_tr**2)/2.0

	!****** fixed effect ******
	integer,parameter :: nf = 2
	real(dp),parameter :: sig_fx = 0.24
	real(dp),parameter :: mu_fx = -(sig_fx**2)/2.0

	!****** social security ******
	real(dp),parameter :: rep_rate = 0.4

	!****** TFP factor level ******
	real(dp),parameter :: tfp80 = 1.0893
	real(dp),parameter :: tfp22 = 1.0893
	real(dp),parameter :: tfp70s= 1.1645
	real(dp),parameter :: tfp80s= 1.0780
	real(dp),parameter :: tfp90s= 1.0179

	!****** TFP factor growth rate ******
	real(dp),parameter :: tfp_gr80 = 1.0134
	real(dp),parameter :: tfp_gr22 = 1.0100
	real(dp),parameter :: tfp_gr70s= 1.0177
	real(dp),parameter :: tfp_gr80s= 1.0228
	real(dp),parameter :: tfp_gr90s= 1.0067

	!****** capital share ******
	real(dp),parameter :: theta80 = 0.3452
	real(dp),parameter :: theta22 = 0.3770
	real(dp),parameter :: theta70s= 0.3512
	real(dp),parameter :: theta80s= 0.3536
	real(dp),parameter :: theta90s= 0.3627

	!****** depreciation rate ******
	real(dp),parameter :: delta80 = 0.0949
	real(dp),parameter :: delta22 = 0.0800
	real(dp),parameter :: delta70s= 0.1019
	real(dp),parameter :: delta80s= 0.0897
	real(dp),parameter :: delta90s= 0.0840

	!****** capital income tax rate ******
	real(dp),parameter :: tax80 = 0.4636
	real(dp),parameter :: tax22 = 0.3980
	real(dp),parameter :: tax70s= 0.4155
	real(dp),parameter :: tax80s= 0.4768
	real(dp),parameter :: tax90s= 0.4492
	
	!****** G/Y ******
	real(dp),parameter :: goy80 = 0.1544
	real(dp),parameter :: goy22 = 0.1500
	real(dp),parameter :: goy70s= 0.1404
	real(dp),parameter :: goy80s= 0.1435
	real(dp),parameter :: goy90s= 0.1482

	!****** demographics ******
	real(dp),parameter :: pop_gr = 1.0
	integer,parameter :: ts = 1980
	integer,parameter :: tc = 2000
	integer,parameter :: tt = 2055
	integer,parameter :: tf = 2200

end module mod_calibration