subroutine all_results(policy,phi,rent,wage,tax_ss,tax_lab,K_agg,H_agg,beq,gov_exp,K_mean_age,H_mean_age,H_hour_age)
!  Purpose:
!  Compute average and variance paths.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  11/05/2006   T. Yamada    Original code
!  11/10/2008   T. Yamada    Revised to Submit JEDC
!  10/20/2010   T. Yamada    Remove wealth inequality
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none

	!****** input ******
	type(policy_function),intent(in) :: policy
	type(phi_function),intent(in) :: phi
	real(dp),intent(in) :: rent,wage,K_agg,H_agg,beq,tax_ss,tax_lab,gov_exp
	real(dp),dimension(js:jf) :: K_mean_age
	real(dp),dimension(js:jr) :: H_mean_age,H_hour_age

	!****** local variables ******
	integer :: i,j,k,l,ia

	!****** profiles ******
	real(dp),dimension(js:jf) :: cons_mean_path
	real(dp),dimension(js:jf) :: erng_mean_path

	!****** variances of log ******
	real(dp) :: var_cons_60,var_erng_60,var_incm_60,var_work_60
	real(dp) :: var_cons_65,var_erng_65,var_incm_65,var_work_65
	real(dp) :: var_cons_70,var_incm_70

	!****** slopes ******
	real(dp) :: yslope,cslope

	!****** labor participation ******
	real(dp),dimension(js:jr) :: labor_part,labor_part_400,labor_part_800

	!****** Frisch elasticity ******
	real(dp) :: frisch,lambda,corr_wh_65,corr_wh_60

	!****** macro variables ******
	real(dp) :: ave_earning,ave_hour_60,ave_hour_65
	real(dp) :: agg_cons,agg_product,agg_save_rate
	!-----------------------------

	!/* consumption, earnings, and income inequality */
	call cons_income_inequality(policy,phi,rent,wage,tax_ss,tax_lab,H_agg,beq,cons_mean_path,erng_mean_path,&
	                          & var_cons_60,var_erng_60,var_incm_60,var_work_60,&
			           & var_cons_65,var_erng_65,var_incm_65,var_work_65,var_cons_70,var_incm_70,&
	                          & yslope,cslope,labor_part,labor_part_400,labor_part_800)

	!/* correlation btwn wage and hours worked */
	call corr_wage_hours(policy,phi,wage,corr_wh_65,corr_wh_60)

	!/* aggregate consumption
	agg_cons = 0.0
	do i = js,jf
		agg_cons = mu(i)*cons_mean_path(i) + agg_cons
	end do

	!/* average earnings and hours worked: 20-65 & 20-60
	ave_earning = 0.0
	ave_hour_65 = 0.0
	do i = js,jr
		ave_earning = (mu(i)/sum(mu(js:jr),dim=1))*erng_mean_path(i) + ave_earning
		ave_hour_65 = (mu(i)/sum(mu(js:jr),dim=1))*H_hour_age(i) + ave_hour_65
	end do
	ave_hour_60 = 0.0
	do i = js,60
		ave_hour_60 = (mu(i)/sum(mu(js:60),dim=1))*H_hour_age(i) + ave_hour_60
	end do

	!/* aggregate output
	agg_product = tfp*(K_agg**theta)*(H_agg**(1.0-theta))

	!/* aggregate saving rate
	agg_save_rate = (agg_product-agg_cons-gov_exp-delta*K_agg)/(agg_product-delta*K_agg)

	!/* Frisch elasticity
	lambda = (1.0-sigma+sigma*gamma)/gamma
	frisch = lambda*((hmax-ave_hour_65)/ave_hour_65)
	open(16,file='Frisch_elasticity.txt')
		write(16,*) "Coefficient of Relative Risk Aversion:        ", 1.0-sigma+sigma*gamma
		write(16,*) "Frisch Elasticity of Labor Supply (20-65):    ", lambda*((hmax-ave_hour_65)/ave_hour_65)
		write(16,*) "Frisch Elasticity of Labor Supply (20-60):    ", lambda*((hmax-ave_hour_60)/ave_hour_60)
		write(16,*) ""
		write(16,*) "Correlation btw Wage and Hours Worked (20-65):", corr_wh_65
		write(16,*) "Correlation btw Wage and Hours Worked (20-60):", corr_wh_60
		write(16,*) ""
		write(16,*) "Frisch Elasticity by Age:"
		do ia = js,jr
			write(16,*) ((1.0-sigma+sigma*gamma)/gamma)*((hmax-H_hour_age(ia))/H_hour_age(ia))
		end do
	close(16)

	!/* output results */
	open(10,file='main_results.txt')
		write(10,*) "K over Y"
		write(10,*) K_agg/agg_product
		write(10,*) ""
		write(10,*) "Factor Prices (Before-Tax and After-Tax Rent, and Wage):"
		write(10,*) rent
		write(10,*) (1.0-tax_cap)*rent
		write(10,*) wage
		write(10,*) ""
		write(10,*) "Aggregate Capital and Labor:"
		write(10,*) K_agg
		write(10,*) H_agg
		write(10,*) ""
		write(10,*) "Average Earnings and Hours Worked:"
		write(10,*) ave_earning
		write(10,*) ave_hour_65
		write(10,*) ave_hour_60
		write(10,*) ""
		write(10,*) "Social Security (Payroll Tax Rate & Replacement Rate):"
		write(10,*) tax_ss
		write(10,*) rep_rate
		write(10,*) ""
		write(10,*) "Tax Rates and Government Expenditure:"
		write(10,*) tax_lab
		write(10,*) tax_cap
		write(10,*) gov_exp
		write(10,*) ""
		write(10,*) "Accidental Bequest:"
		write(10,*) beq
		write(10,*) ""
		write(10,*) "Per Capita Output, Consumption and Saving Rate:"
		write(10,*) agg_product
		write(10,*) agg_cons
		write(10,*) agg_save_rate
		write(10,*) ""
		write(10,*) "Total Factor Productivity:"
		write(10,*) tfp
		write(10,*) tfp_gr
		write(10,*) ""
		write(10,*) "The Number of Grid:"
		write(10,*) na,nz,nt,nf,nd
		write(10,*) ""
		write(10,*) "Flags:"
		write(10,*) "Year:      ", flag_year
		write(10,*) "Population:", flag_pop
		write(10,*) ""
		write(10,*) "Preference Parameters:"
		write(10,*) "beta:  ", beta
		write(10,*) "gamma: ", gamma
		write(10,*) "sigma: ", sigma
		write(10,*) "h max: ", hmax
		write(10,*) "psi:   ", psi
		write(10,*) ""
		write(10,*) "Replacement Rate: ", rep_rate
		write(10,*) "Capital Share:    ", theta
		write(10,*) "Depreciation Rate:", delta
		write(10,*) ""
		write(10,*) "rho:   ", rho
		write(10,*) "std_eps:", std_eps20
		write(10,*) "std_pr:", std_pr
		write(10,*) "sig_tr:", sig_tr
		write(10,*) "sig_fx:", sig_fx
	close(10)

	open(11,file='main_Excel.txt')
		write(11,*) K_agg/agg_product
		write(11,*) gov_exp/agg_product
		write(11,*) (1.0-tax_cap)*rent
		write(11,*) wage
		write(11,*) tax_ss
		write(11,*) tax_lab
		write(11,*) K_agg
		write(11,*) H_agg
		write(11,*) ave_hour_65
		write(11,*) ave_hour_60
		write(11,*) var_cons_60
		write(11,*) var_erng_60
		write(11,*) var_incm_60
		write(11,*) var_work_60
		write(11,*) var_cons_65
		write(11,*) var_erng_65
		write(11,*) var_incm_65
		write(11,*) var_work_65
		write(11,*) var_cons_70
		write(11,*) var_incm_70
		write(11,*) frisch
		write(11,*) corr_wh_65
		write(11,*) corr_wh_60
		write(11,*) yslope
		write(11,*) cslope
	close(11)

	!/* average capital and labor path */
	open(21,file='profile_capital.out')
	open(22,file='profile_hours.out')
		do i = js,jf
			write(21,*) K_mean_age(i)
		end do
		do i = js,jr
			write(22,*) H_hour_age(i)
		end do
	close(21)
	close(22)

	!/* consumption and earning profile */
	open(23,file='profile_cons.out')
	open(24,file='profile_earning.out')
		do i = js,jf
			write(23,*) cons_mean_path(i)
			write(24,*) erng_mean_path(i)
		end do
	close(23)
	close(24)

	!/* labor participation rate */
!	open(31,file='labor_participation_rate.out')
!	open(32,file='labor_participation_rate_400.out')
!	open(33,file='labor_participation_rate_800.out')
!		do i = js,jr
!			write(31,*) labor_part(i)
!			write(32,*) labor_part_400(i)
!			write(33,*) labor_part_800(i)
!		end do
!	close(31)
!	close(32)
!	close(33)

end subroutine all_results


subroutine cons_income_inequality(policy,phi,rent,wage,tax_ss,tax_lab,H_agg,beq,cons_mean_path,erng_mean_path,&
                                 & var_cons_60,var_erng_60,var_incm_60,var_work_60,&
                                 & var_cons_65,var_erng_65,var_incm_65,var_work_65,var_cons_70,var_incm_70,&
                                 & yslope,cslope,labor_part,labor_part_400,labor_part_800)
!  Purpose:
!  Averages and variances of log-consumption and earnings.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  02/06/2008   T. Yamada    Original code
!  11/08/2008   T. Yamada    Revised to Submit JEDC
!  10/21/2010   T. Yamada    Add slope differences etc.
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_interpolation
 	implicit none

	!****** input ******
	type(policy_function),intent(in) :: policy
	type(phi_function),intent(in) :: phi
	real(dp),intent(in) :: rent,wage,tax_ss,tax_lab,H_agg,beq

	!****** output ******
	real(dp),intent(out),dimension(js:jf) :: cons_mean_path
	real(dp),intent(out),dimension(js:jf) :: erng_mean_path
	real(dp),intent(out) :: var_cons_60,var_erng_60,var_incm_60,var_work_60
	real(dp),intent(out) :: var_cons_65,var_erng_65,var_incm_65,var_work_65
	real(dp),intent(out) :: var_cons_70,var_incm_70
	real(dp),intent(out) :: yslope,cslope
	real(dp),intent(out),dimension(js:jr) :: labor_part,labor_part_400,labor_part_800

	!****** local variables ******
	integer :: i,j,k,l,ia,status
	real(dp) :: wealth
	real(dp) :: sqr_erng_60,sqr_incm_60,sqr_work_60
	real(dp) :: avg_erng_60,avg_incm_60,avg_work_60
	real(dp) :: sqr_erng_65,sqr_incm_65,sqr_work_65
	real(dp) :: avg_erng_65,avg_incm_65,avg_work_65
	real(dp) :: avg_cons_60,avg_cons_65
	real(dp) :: sqr_cons_60,sqr_cons_65
	real(dp) :: avg_cons_70,sqr_cons_70
	real(dp) :: avg_incm_70,sqr_incm_70
	real(dp),dimension(0:nd) :: cdf,cdf_value
	real(dp),dimension(js:jf) :: avg_cons,avg_erng,avg_incm,avg_work
	real(dp),dimension(js:jf) :: sqr_cons,sqr_erng,sqr_incm,sqr_work
	real(dp),dimension(js:jf) :: var_cons,var_erng,var_incm,var_work
	real(dp),dimension(js:jr) :: avg_erng_part,var_erng_part
	real(dp),dimension(js:jf) :: zlc,zly,zli,zlh,zly_part
	real(dp),allocatable,dimension(:,:,:,:,:) :: log_cons1,log_incm1,log_erng1,log_work1,log_erng_part
	real(dp),allocatable,dimension(:,:,:,:,:) :: saving1,hours,cons1,income1,earning1
	real(dp),allocatable,dimension(:,:,:) :: log_cons2,log_incm2,log_erng2
	real(dp),allocatable,dimension(:,:,:) :: saving2,cons2,income2,earning2
	!-----------------------------

	write(*,*) ""
	write(*,*) "-+-+- Consumption, Earnings, and Income Inequality -+-+-"

	allocate(log_cons1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(log_incm1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_erng1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_work1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_cons2(nd,nf,jr+1:jf),STAT=status)
	allocate(log_incm2(nd,nf,jr+1:jf),STAT=status)
	allocate(log_erng2(nd,nf,jr+1:jf),STAT=status)
	allocate(log_erng_part(nd,nz,nt,nf,js:jr),STAT=status)
	log_cons1 = 0.0
	log_cons2 = 0.0
	log_incm1 = 0.0
	log_incm2 = 0.0
	log_erng1 = 0.0
	log_erng2 = 0.0
	log_work1 = 0.0
	log_erng_part = 0.0

	allocate(saving1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(saving2(nd,nf,jr+1:jf),STAT=status)
	allocate(cons1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(cons2(nd,nf,jr+1:jf),STAT=status)
	allocate(income1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(income2(nd,nf,jr+1:jf),STAT=status)
	allocate(earning1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(earning2(nd,nf,jr+1:jf),STAT=status)
	allocate(hours(nd,nz,nt,nf,js:jr),STAT=status)
	saving1 = 0.0
	saving2 = 0.0
	cons1 = 0.0
	cons2 = 0.0
	income1 = 0.0
	income2 = 0.0
	earning1 = 0.0
	earning2 = 0.0
	hours = 0.0

	!/* re-compute policy function over finer grids */
	call finer_policy(policy,saving1,hours,saving2)

	!/* consumption, earnings, and disposable income */
	labor_part = 0.0
	labor_part_400 = 0.0
	labor_part_800 = 0.0
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							earning1(i,j,k,l,ia) = wage*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)*hours(i,j,k,l,ia)
							income1(i,j,k,l,ia)  = rent*(grcap2(i)+beq) + earning1(i,j,k,l,ia)
							wealth               = (1.0+(1.0-tax_cap)*rent)*(grcap2(i)+beq) + (1.0-tax_ss-tax_lab)*earning1(i,j,k,l,ia)
							cons1(i,j,k,l,ia)    = wealth - tfp_gr*saving1(i,j,k,l,ia)
							if (hours(i,j,k,l,ia) > 0.0) then
								labor_part(ia) = phi%y(i,j,k,l,ia) + labor_part(ia)
							end if
							if (hours(i,j,k,l,ia) > 400.0) then
								labor_part_400(ia) = phi%y(i,j,k,l,ia) + labor_part_400(ia)
							end if
							if (hours(i,j,k,l,ia) > 800.0) then
								labor_part_800(ia) = phi%y(i,j,k,l,ia) + labor_part_800(ia)
							end if
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					earning2(i,l,ia) = rep_rate*wage*H_agg*shock_fx(l)
					income2(i,l,ia)  = rent*(grcap2(i)+beq) + earning2(i,l,ia)
					wealth           = (1.0+(1.0-tax_cap)*rent)*(grcap2(i)+beq) + earning2(i,l,ia)
					cons2(i,l,ia)    = wealth - tfp_gr*saving2(i,l,ia)
				end do
			end do
		end select
	end do
	deallocate(saving1)
	deallocate(saving2)

	!/* log of consumption, earning, disposable income and hours */
	zlc = 0.0
	zly = 0.0
	zli = 0.0
	zlh = 0.0
	do ia = js,jf
		select case(ia)
		case(js:jr)
			do l = 1,nf
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							if (cons1(i,j,k,l,ia) > 0.0) then
								log_cons1(i,j,k,l,ia) = log(cons1(i,j,k,l,ia))
							else
								log_cons1(i,j,k,l,ia) = 0.0
								zlc(ia) = phi%y(i,j,k,l,ia) + zlc(ia)
							end if
							if (earning1(i,j,k,l,ia) > 0.0) then
								log_erng1(i,j,k,l,ia) = log(earning1(i,j,k,l,ia))
							else
								log_erng1(i,j,k,l,ia) = 0.0
								zly(ia) = phi%y(i,j,k,l,ia) + zly(ia)
							end if
							!/* hours inequality: exclude part-time */
							if (earning1(i,j,k,l,ia)>0.0 .and. hours(i,j,k,l,ia)>100.0) then
								log_erng_part(i,j,k,l,ia) = log(earning1(i,j,k,l,ia))
							else
								log_erng_part(i,j,k,l,ia) = 0.0
								zly_part(ia) = phi%y(i,j,k,l,ia) + zly_part(ia)
							end if
							!-----------------------------------------
							if (income1(i,j,k,l,ia) > 0.0) then
								log_incm1(i,j,k,l,ia) = log(income1(i,j,k,l,ia))
							else
								log_incm1(i,j,k,l,ia) = 0.0
								zli(ia) = phi%y(i,j,k,l,ia) + zli(ia)
							end if
							if (hours(i,j,k,l,ia) > 0.0) then
								log_work1(i,j,k,l,ia) = log(hours(i,j,k,l,ia))
							else
								log_work1(i,j,k,l,ia) = 0.0
								zlh(ia) = phi%y(i,j,k,l,ia) + zlh(ia)
							end if
						end do
					end do
				end do
			end do
		case(jr+1:jf)
			do l = 1,nf
				do i = 1,nd
					if (cons2(i,l,ia) > 0) then
						log_cons2(i,l,ia) = log(cons2(i,l,ia))
					else
						log_cons2(i,l,ia) = 0.0
						zlc(ia) = phi%y(i,j,k,l,ia) + zlc(ia)
					end if
					if (earning2(i,l,ia) > 0) then
						log_erng2(i,l,ia) = log(earning2(i,l,ia))
					else
						log_erng2(i,l,ia) = 0.0
						zly(ia) = phi%y(i,j,k,l,ia) + zly(ia)
					end if
					if (income2(i,l,ia) > 0) then
						log_incm2(i,l,ia) = log(income2(i,l,ia))
					else
						log_incm2(i,l,ia) = 0.0
						zli(ia) = phi%y(i,j,k,l,ia) + zli(ia)
					end if
				end do
			end do
		end select
	end do

	!/* variances of log-consumption etc. by age */
	sqr_cons = 0.0
	sqr_erng = 0.0
	sqr_incm = 0.0
	sqr_work = 0.0
	avg_cons = 0.0
	avg_erng = 0.0
	avg_incm = 0.0
	avg_work = 0.0
	do l = 1,nf
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							sqr_cons(ia) = phi%y(i,j,k,l,ia)*(log_cons1(i,j,k,l,ia)**2) + sqr_cons(ia)
							sqr_erng(ia) = phi%y(i,j,k,l,ia)*(log_erng1(i,j,k,l,ia)**2) + sqr_erng(ia)
							sqr_incm(ia) = phi%y(i,j,k,l,ia)*(log_incm1(i,j,k,l,ia)**2) + sqr_incm(ia)
							sqr_work(ia) = phi%y(i,j,k,l,ia)*(log_work1(i,j,k,l,ia)**2) + sqr_work(ia)
							avg_cons(ia) = phi%y(i,j,k,l,ia)*log_cons1(i,j,k,l,ia) + avg_cons(ia)
							avg_erng(ia) = phi%y(i,j,k,l,ia)*log_erng1(i,j,k,l,ia) + avg_erng(ia)
							avg_incm(ia) = phi%y(i,j,k,l,ia)*log_incm1(i,j,k,l,ia) + avg_incm(ia)
							avg_work(ia) = phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work(ia)
						end do
					end do
				end do
				sqr_cons(ia) = sqr_cons(ia) / (1.0-zlc(ia))
				sqr_erng(ia) = sqr_erng(ia) / (1.0-zly(ia))
				sqr_incm(ia) = sqr_incm(ia) / (1.0-zli(ia))
				sqr_work(ia) = sqr_work(ia) / (1.0-zlh(ia))
				avg_cons(ia) = avg_cons(ia) / (1.0-zlc(ia))
				avg_erng(ia) = avg_erng(ia) / (1.0-zly(ia))
				avg_incm(ia) = avg_incm(ia) / (1.0-zli(ia))
				avg_work(ia) = avg_work(ia) / (1.0-zlh(ia))
			case(jr+1:jf)
				do i = 1,nd
					sqr_cons(ia) = phi%o(i,l,ia)*(log_cons2(i,l,ia)**2) + sqr_cons(ia)
					sqr_erng(ia) = phi%o(i,l,ia)*(log_erng2(i,l,ia)**2) + sqr_erng(ia)
					sqr_incm(ia) = phi%o(i,l,ia)*(log_incm2(i,l,ia)**2) + sqr_incm(ia)
					avg_cons(ia) = phi%o(i,l,ia)*log_cons2(i,l,ia) + avg_cons(ia)
					avg_erng(ia) = phi%o(i,l,ia)*log_erng2(i,l,ia) + avg_erng(ia)
					avg_incm(ia) = phi%o(i,l,ia)*log_incm2(i,l,ia) + avg_incm(ia)
				end do
				sqr_cons(ia) = sqr_cons(ia) / (1.0-zlc(ia))
				sqr_erng(ia) = sqr_erng(ia) / (1.0-zly(ia))
				sqr_incm(ia) = sqr_incm(ia) / (1.0-zli(ia))
				avg_cons(ia) = avg_cons(ia) / (1.0-zlc(ia))
				avg_erng(ia) = avg_erng(ia) / (1.0-zly(ia))
				avg_incm(ia) = avg_incm(ia) / (1.0-zli(ia))
			end select
		end do
	end do
	var_cons = 0.0
	var_erng = 0.0
	var_incm = 0.0
	var_work = 0.0
	do ia = js,jf
		var_cons(ia) = sqr_cons(ia) - avg_cons(ia)**2
		var_erng(ia) = sqr_erng(ia) - avg_erng(ia)**2
		var_incm(ia) = sqr_incm(ia) - avg_incm(ia)**2
	end do
	do ia = js,jr
		var_work(ia) = sqr_work(ia) - avg_work(ia)**2
	end do

	!/* output results */
	open(10,file='profile_var_cons.out')
	open(11,file='profile_var_income.out')
	open(12,file='profile_var_earning.out')
	open(13,file='profile_var_hours.out')
		do i = js,jf
			write(10,*) var_cons(i)
			write(11,*) var_incm(i)
			write(12,*) var_erng(i)
		end do
		do i = js,jr
			write(13,*) var_work(i)
		end do
	close(10)
	close(11)
	close(12)
	close(13)

	!/* economy-wide inequality */
	sqr_cons_60 = 0.0
	sqr_erng_60 = 0.0
	sqr_incm_60 = 0.0
	sqr_work_60 = 0.0
	avg_cons_60 = 0.0
	avg_erng_60 = 0.0
	avg_incm_60 = 0.0
	avg_work_60 = 0.0
	sqr_cons_65 = 0.0
	sqr_erng_65 = 0.0
	sqr_incm_65 = 0.0
	sqr_work_65 = 0.0
	avg_cons_65 = 0.0
	avg_erng_65 = 0.0
	avg_incm_65 = 0.0
	avg_work_65 = 0.0
	sqr_cons_70 = 0.0
	sqr_incm_70 = 0.0
	avg_cons_70 = 0.0
	avg_incm_70 = 0.0
	do ia = js,60
		sqr_cons_60 = mu(ia)/sum(mu(js:60),dim=1)*sqr_cons(ia) + sqr_cons_60
		sqr_erng_60 = mu(ia)/sum(mu(js:60),dim=1)*sqr_erng(ia) + sqr_erng_60
		sqr_incm_60 = mu(ia)/sum(mu(js:60),dim=1)*sqr_incm(ia) + sqr_incm_60
		sqr_work_60 = mu(ia)/sum(mu(js:60),dim=1)*sqr_work(ia) + sqr_work_60
		avg_cons_60 = mu(ia)/sum(mu(js:60),dim=1)*avg_cons(ia) + avg_cons_60
		avg_erng_60 = mu(ia)/sum(mu(js:60),dim=1)*avg_erng(ia) + avg_erng_60
		avg_incm_60 = mu(ia)/sum(mu(js:60),dim=1)*avg_incm(ia) + avg_incm_60
		avg_work_60 = mu(ia)/sum(mu(js:60),dim=1)*avg_work(ia) + avg_work_60
	end do
	do ia = js,65
		sqr_cons_65 = mu(ia)/sum(mu(js:65),dim=1)*sqr_cons(ia) + sqr_cons_65
		sqr_erng_65 = mu(ia)/sum(mu(js:65),dim=1)*sqr_erng(ia) + sqr_erng_65
		sqr_incm_65 = mu(ia)/sum(mu(js:65),dim=1)*sqr_incm(ia) + sqr_incm_65
		sqr_work_65 = mu(ia)/sum(mu(js:65),dim=1)*sqr_work(ia) + sqr_work_65
		avg_cons_65 = mu(ia)/sum(mu(js:65),dim=1)*avg_cons(ia) + avg_cons_65
		avg_erng_65 = mu(ia)/sum(mu(js:65),dim=1)*avg_erng(ia) + avg_erng_65
		avg_incm_65 = mu(ia)/sum(mu(js:65),dim=1)*avg_incm(ia) + avg_incm_65
		avg_work_65 = mu(ia)/sum(mu(js:65),dim=1)*avg_work(ia) + avg_work_65
	end do
	do ia = js,70
		sqr_cons_70 = mu(ia)/sum(mu(js:70),dim=1)*sqr_cons(ia) + sqr_cons_70
		sqr_incm_70 = mu(ia)/sum(mu(js:70),dim=1)*sqr_incm(ia) + sqr_incm_70
		avg_cons_70 = mu(ia)/sum(mu(js:70),dim=1)*avg_cons(ia) + avg_cons_70
		avg_incm_70 = mu(ia)/sum(mu(js:70),dim=1)*avg_incm(ia) + avg_incm_70
	end do
	var_cons_60 = sqr_cons_60 - avg_cons_60**2
	var_cons_65 = sqr_cons_65 - avg_cons_65**2
	var_cons_70 = sqr_cons_70 - avg_cons_70**2
	var_incm_60 = sqr_incm_60 - avg_incm_60**2
	var_incm_65 = sqr_incm_65 - avg_incm_65**2
	var_incm_70 = sqr_incm_70 - avg_incm_70**2
	var_erng_60 = sqr_erng_60 - avg_erng_60**2
	var_erng_65 = sqr_erng_65 - avg_erng_65**2
	var_work_60 = sqr_work_60 - avg_work_60**2
	var_work_65 = sqr_work_65 - avg_work_65**2
	write(*,*) "Variance of Log-Consumption: ", var_cons_65
	write(*,*) "Variance of Log-Earning:     ", var_erng_65
	open(14,file='inequality_of_economy.txt')
		write(14,*) "Variance of Log-Consumption 20-60: ", var_cons_60
		write(14,*) "Variance of Log-Earnings 20-60:    ", var_erng_60
		write(14,*) "Variance of Log-Income 20-60:      ", var_incm_60
		write(14,*) "Variance of Log-Hours Worked 20-60:", var_work_60
		write(14,*) ""
		write(14,*) "Variance of Log-Consumption 20-65: ", var_cons_65
		write(14,*) "Variance of Log-Earnings 20-65:    ", var_erng_65
		write(14,*) "Variance of Log-Income 20-65:      ", var_incm_65
		write(14,*) "Variance of Log-Hours Worked 20-65:", var_work_65
		write(14,*) ""
		write(14,*) "Variance of Log-Consumption 20-70: ", var_cons_70
		write(14,*) "Variance of Log-Income 20-70:      ", var_incm_70
	close(14)

!ADD NEW [Feb 27, 2011]
	!/* Earning inequality that exclude extremely low hours
	avg_erng_part = 0.0
	var_erng_part = 0.0
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						avg_erng_part(ia) = phi%y(i,j,k,l,ia)*log_erng_part(i,j,k,l,ia) + avg_erng_part(ia)
					end do
				end do
			end do
		end do
	end do
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						var_erng_part(ia) = phi%y(i,j,k,l,ia)*((log_erng_part(i,j,k,l,ia)-avg_erng_part(ia))**2) + var_erng_part(ia)
					end do
				end do
			end do
		end do
		var_erng_part(ia) = var_erng_part(ia) / (1.0-zly_part(ia))
	end do
	open(15,file='profile_var_earning_part.out')
		do i = js,jr
			write(15,*) var_erng_part(i)
		end do
	close(15)

	!/* consumption and before-tax earnings profiles */
	cons_mean_path = 0.0
	erng_mean_path = 0.0
	do l = 1,nf
		do ia = js,jf
			select case(ia)
			case(js:jr)
				do k = 1,nt
					do j = 1,nz
						do i = 1,nd
							cons_mean_path(ia) = phi%y(i,j,k,l,ia)*cons1(i,j,k,l,ia) + cons_mean_path(ia)
							erng_mean_path(ia) = phi%y(i,j,k,l,ia)*earning1(i,j,k,l,ia) + erng_mean_path(ia)
						end do
					end do
				end do
			case(jr+1:jf)
				do i = 1,nd
					cons_mean_path(ia) = phi%o(i,l,ia)*cons2(i,l,ia) + cons_mean_path(ia)
					erng_mean_path(ia) = phi%o(i,l,ia)*earning2(i,l,ia) + erng_mean_path(ia)
				end do
			end select
		end do
	end do

	!/* slopes of earnings and consumption profile
	yslope = (sum(erng_mean_path(45:55))/11.0) / (sum(erng_mean_path(25:35))/11.0)
	cslope = (sum(cons_mean_path(45:55))/11.0) / (sum(cons_mean_path(25:35))/11.0)
	write(*,*) "Slope of Earnings:   ", yslope
	write(*,*) "Slope of Consumption:", cslope
	write(*,*) ""
	open(15,file='slope_of_profile.txt')
		write(15,*) "Slope of Earnings: ave(45-55)/ave(25-35)", yslope
		write(15,*) "Slope of Consumption: ave(45-54)/ave(25-34)", cslope
	close(15)

	deallocate(log_cons1)
	deallocate(log_cons2)
	deallocate(log_incm1)
	deallocate(log_incm2)
	deallocate(log_erng1)
	deallocate(log_erng2)
	deallocate(log_work1)
	deallocate(cons1)
	deallocate(cons2)
	deallocate(income1)
	deallocate(income2)
	deallocate(earning1)
	deallocate(earning2)
	deallocate(hours)

end subroutine cons_income_inequality


subroutine corr_wage_hours(policy,phi,wage,corr_wh_65,corr_wh_60)
!  Purpose:
!  Correlation between wage and hours.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  10/21/2010   T. Yamada    Original code
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	use mod_interpolation
 	implicit none

	!****** input ******
	type(policy_function),intent(in) :: policy
	type(phi_function),intent(in) :: phi
	real(dp),intent(in) :: wage

	!****** output ******
	real(dp),intent(out) :: corr_wh_65,corr_wh_60

	!****** local variables ******
	integer :: i,j,k,l,ia,status
	real(dp) :: ww
	real(dp) :: avg_work_65,avg_wage_65
	real(dp) :: var_work_65,var_wage_65,cov_wh_65
	real(dp) :: avg_work_60,avg_wage_60
	real(dp) :: var_work_60,var_wage_60,cov_wh_60
	real(dp),dimension(js:jf) :: avg_work,avg_wage
	real(dp),dimension(js:jf) :: var_work,var_wage,cov_wh,corr_wh
	real(dp),allocatable,dimension(:,:,:,:,:) :: log_work1,log_wage1
	real(dp),allocatable,dimension(:,:,:,:,:) :: saving1,hours
	real(dp),allocatable,dimension(:,:,:) :: saving2
	!-----------------------------

	allocate(log_work1(nd,nz,nt,nf,js:jr),STAT=status)
	allocate(log_wage1(nd,nz,nt,nf,js:jr),STAT=status)
	log_work1 = 0.0
	log_wage1 = 0.0
	allocate(saving1(nd,nz,nt,nf,js:jf),STAT=status)
	allocate(saving2(nd,nf,jr+1:jf),STAT=status)
	allocate(hours(nd,nz,nt,nf,js:jr),STAT=status)
	saving1 = 0.0
	saving2 = 0.0
	hours = 0.0

	!/* re-compute policy function over finer grids */
	call finer_policy(policy,saving1,hours,saving2)
	deallocate(saving1)
	deallocate(saving2)

	!/* log-wage and log-hours
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						ww = wage*kappa(ia)*endow(j,ia)*shock_tr(k)*shock_fx(l)
						if (hours(i,j,k,l,ia) > 0.0) then
							log_wage1(i,j,k,l,ia) = log(ww)
							log_work1(i,j,k,l,ia) = log(hours(i,j,k,l,ia))
						else
							log_wage1(i,j,k,l,ia) = 0.0
							log_work1(i,j,k,l,ia) = 0.0
						end if
					end do
				end do
			end do
		end do
	end do

	!/* mean of log-wage and log-hours
	avg_wage = 0.0
	avg_work = 0.0
	avg_wage_65 = 0.0
	avg_work_65 = 0.0
	avg_wage_60 = 0.0
	avg_work_60 = 0.0
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						avg_wage(ia) = phi%y(i,j,k,l,ia)*log_wage1(i,j,k,l,ia) + avg_wage(ia)
						avg_work(ia) = phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work(ia)
						avg_wage_65 = (mu(ia)/sum(mu(js:65),dim=1))*phi%y(i,j,k,l,ia)*log_wage1(i,j,k,l,ia) + avg_wage_65
						avg_work_65 = (mu(ia)/sum(mu(js:65),dim=1))*phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work_65
						if (ia <= 60) then
							avg_wage_60 = (mu(ia)/sum(mu(js:60),dim=1))*phi%y(i,j,k,l,ia)*log_wage1(i,j,k,l,ia) + avg_wage_60
							avg_work_60 = (mu(ia)/sum(mu(js:60),dim=1))*phi%y(i,j,k,l,ia)*log_work1(i,j,k,l,ia) + avg_work_60
						end if
					end do
				end do
			end do
		end do
	end do

	!/* correlation btw log-wage and log-hours */
	var_wage = 0.0
	var_work = 0.0
	cov_wh   = 0.0
	var_wage_65 = 0.0
	var_work_65 = 0.0
	cov_wh_65 = 0.0
	var_wage_60 = 0.0
	var_work_60 = 0.0
	cov_wh_60 = 0.0
	do ia = js,jr
		do l = 1,nf
			do k = 1,nt
				do j = 1,nz
					do i = 1,nd
						var_wage(ia) = phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage(ia))**2) + var_wage(ia)
						var_work(ia) = phi%y(i,j,k,l,ia)*((log_work1(i,j,k,l,ia)-avg_work(ia))**2) + var_work(ia)
						cov_wh(ia)   = phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage(ia))*(log_work1(i,j,k,l,ia)-var_work(ia))) + cov_wh(ia)
						var_wage_65 = (mu(ia)/sum(mu(js:65),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_65)**2) + var_wage_65
						var_work_65 = (mu(ia)/sum(mu(js:65),dim=1))*phi%y(i,j,k,l,ia)*((log_work1(i,j,k,l,ia)-avg_work_65)**2) + var_work_65
						cov_wh_65   = (mu(ia)/sum(mu(js:65),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_65)*(log_work1(i,j,k,l,ia)-avg_work_65)) + cov_wh_65
						if (ia <= 60) then
							var_wage_60 = (mu(ia)/sum(mu(js:60),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_60)**2) + var_wage_60
							var_work_60 = (mu(ia)/sum(mu(js:60),dim=1))*phi%y(i,j,k,l,ia)*((log_work1(i,j,k,l,ia)-avg_work_60)**2) + var_work_60
							cov_wh_60   = (mu(ia)/sum(mu(js:60),dim=1))*phi%y(i,j,k,l,ia)*((log_wage1(i,j,k,l,ia)-avg_wage_60)*(log_work1(i,j,k,l,ia)-avg_work_60)) + cov_wh_60
						end if
					end do
				end do
			end do
		end do
	end do

	!/* corr(w,h): age-dependent
	do ia = js,jr
		corr_wh(ia) = sqrt((cov_wh(ia)**2)/(var_wage(ia)*var_work(ia)))
	end do
	corr_wh_65 = sqrt((cov_wh_65**2)/(var_wage_65*var_work_65))
	corr_wh_60 = sqrt((cov_wh_60**2)/(var_wage_60*var_work_60))

	!/* output results
	open(18,file='profile_corr_wh.out')
		do i = js,jr
			write(18,*) corr_wh(i)
		end do
	close(18)

	deallocate(log_work1)
	deallocate(log_wage1)
	deallocate(hours)

end subroutine corr_wage_hours