module mod_calibration
!  Purpose:
!  Collects calibrated parameters.
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  10/20/2006   T. Yamada    Original code
!  01/25/2009   T. Yamada    Revised to submit JEDC
!  11/28/2010   T. Yamada    Reviesd to resubmit JEDC
	use nrtype,only:dp
	implicit none

	!****** preference ******
	real(dp),parameter :: beta = 0.984
	real(dp),parameter :: gamma = 2.0
	real(dp),parameter :: sigma = 0.52
	real(dp),parameter :: psi = 0.0

	!****** lifecycle ******
	integer,parameter :: js = 20
	integer,parameter :: jr = 65
	integer,parameter :: jf = 100

	!****** labor endowment ******
	integer,parameter :: nz = 7
	real(dp),parameter :: rho = 0.97 !0.98
	real(dp),parameter :: mu_z = 0.0
	real(dp),parameter :: std_eps20 = 0.06
	real(dp),parameter :: std_pr = 0.0005 !0.0004

	!****** transitory shocks ******
	integer,parameter :: nt = 2
	real(dp),parameter :: sig_tr = 0.06 !0.10
	real(dp),parameter :: mu_tr = -(sig_tr**2)/2.0

	!****** fixed effect ******
	integer,parameter :: nf = 2
	real(dp),parameter :: sig_fx = 0.24
	real(dp),parameter :: mu_fx = -(sig_fx**2)/2.0

	!****** TFP level ******
	real(dp),parameter :: tfp = 1.0893

	!****** averages in 1980-2000 ******
	real(dp),parameter :: tfp_gr_con = 1.017
	real(dp),parameter :: theta_con = 0.3581
	real(dp),parameter :: delta_con = 0.0864
	real(dp),parameter :: tax_cap_con = 0.4628
	real(dp),parameter :: goy_con = 0.1459

	!****** finite steady state ******
	real(dp),parameter :: tfp_gr_fin = 1.010
	real(dp),parameter :: theta_fin = 0.377
	real(dp),parameter :: delta_fin = 0.080
	real(dp),parameter :: tax_cap_fin = 0.398
	real(dp),parameter :: goy_fin = 0.1500

	!****** social security system ******
	real(dp),parameter :: rep_rate = 0.4

	!****** demographic change ******
	real(dp),parameter :: pop_gr = 1.00
	integer,parameter :: ts = 1980
	integer,parameter :: tc = 2000
	integer,parameter :: tt = 2055
	integer,parameter :: tf = 2200

end module mod_calibration