subroutine demographics
!  Purpose:
!  Load survival probabilities and calculate demographic structure.
!
!  Data source:
!  the National Institute of Population and Social Security Research
!
!  Record of revisions:
!     Date      Programmer   Description of change
!  ==========   ==========   =====================
!  01/26/2007   T. Yamada    Original code
!  11/20/2007   T. Yamada    Updated to h18 version
	use nrtype,only:dp
	use mod_calibration
	use mod_variables
	implicit none

	!****** local variables ******
	integer :: i,j,status
	real(dp),allocatable,dimension(:) :: surv_all
	real(dp),allocatable,dimension(:,:) :: surv_1980_2055,mu_tran
	real(dp),dimension(ts:tt-1) :: pop_growth_1980_2054
	real(dp),dimension(tt:2065) :: pop_growth_2055_2065
	real(dp),dimension(0:jf) :: mu_1980,nu
	real(dp),dimension(0:jf,0:jf) :: GAM
!	real(dp),dimension(ts:tf) :: pop_0_100,pop_20_100
!	real(dp),dimension(ts:tf) :: pop_0_19,pop_20_65,pop_66_100
!	real(dp),dimension(ts:tf) :: pop_20_35,pop_36_50,pop_51_65
	!-----------------------------

	allocate(surv_all(jf*(tt-ts+1)),STAT=status)
	allocate(surv_1980_2055(0:jf,ts:tt),STAT=status)
	allocate(mu_tran(0:jf,ts:tf),STAT=status)
	surv_all       = 0.0
	surv_1980_2055 = 0.0
	mu_tran        = 0.0

	!/* survival probability from 1980 to 2055
	open(10,file='surv_1980_2055.in')
		read(10,*) surv_all
	close(10)
	do i = ts,tt
		surv_1980_2055(0:jf-1,i) = surv_all((i-ts)*jf+1:jf*(i-ts+1))
	end do
	surv_1980_2055(jf,:) = 0.0

	!/* population dynamics */
	select case (flag_pop)
	case(1)

		!/* use actual age distribution
		open(11,file='pop_1980.in')
			read(11,*) mu_1980
		close(11)
		mu_tran(:,ts) = mu_1980

		!/* fertility rate from 1980 to 2054: medium variant
		open(12,file='pop_growth_1980_2054_medium.in')
			read(12,*) pop_growth_1980_2054
		close(12)

		!/* linear convergence of population growth rate from 2055 to 2065
		pop_growth_2055_2065(2065) = 1.0
		do i = 2055,2064,1
			pop_growth_2055_2065(i) = pop_growth_1980_2054(2054) + ((pop_growth_2055_2065(2065)-pop_growth_1980_2054(2054))/real(2064-2054))*(i-2054)
		end do

		!/* population dynamics from 1980 to 2200
		do i = ts,tf-1
			GAM = 0.0
			select case(i)
			!/* 1980-2054
			case(1:tt-1)
				do j = 1,jf
					GAM(j,j-1) = surv_1980_2055(j,i)
				end do
				GAM(0,0)       = pop_growth_1980_2054(i)
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2055-2065
			case(tt:2065)
				do j = 1,jf
					GAM(j,j-1) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = pop_growth_2055_2065(i)
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2066-2200
			case(2066:)
				do j = 1,jf
					GAM(j,j-1) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = 1.0
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			end select
		end do

	case(2)

		!/* use the same survival probability in 2000
		do i = ts,tt
			surv_1980_2055(:,i) = surv_1980_2055(:,tc)
		end do

		!/* initial population distribution
		nu(0) = 1.0
		do j = 0,jf-1
			nu(j+1) = surv_1980_2055(j,tc)/pop_gr*nu(j)
		end do
		do j = 0,jf
			mu_tran(j,ts) = nu(j)/sum(nu(0:jf))
		end do

		!/* population dynamics
		do i = ts,tf-1
			GAM = 0.0
			select case(i)
			!/* 1980-2054
			case(ts:tt-1)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,i)
				end do
				GAM(0,0)       = pop_gr
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2055-2065
			case(tt:2065)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = pop_gr
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			!/* 2066-2200
			case(2066:tf)
				do j = 0,jf-1
					GAM(j+1,j) = surv_1980_2055(j,tt)
				end do
				GAM(0,0)       = pop_gr
				mu_tran(:,i+1) = matmul(GAM,mu_tran(:,i))
			end select
		end do

	end select

	!/* population dynamics
	do i = ts,tf
		mu(:,i) = mu_tran(js:jf,i)/sum(mu_tran(js:jf,i),dim=1)
		frac_ret(i) = sum(mu_tran(jr+1:jf,i),dim=1) / sum(mu_tran(js:jf,i),dim=1)
![REMOVE LATER]
!		pop_0_100(i) = sum(mu_tran(0:jf,i),dim=1)
!		pop_0_19(i) = sum(mu_tran(0:js-1,i),dim=1)
!		pop_20_35(i) = sum(mu_tran(20:35,i),dim=1)
!		pop_36_50(i) = sum(mu_tran(36:50,i),dim=1)
!		pop_51_65(i) = sum(mu_tran(51:65,i),dim=1)
!		pop_66_100(i) = sum(mu_tran(jr+1:jf,i),dim=1)
!		pop_20_65(i) = sum(mu_tran(js:jr,i),dim=1)
!		pop_20_100(i) = sum(mu_tran(js:jf,i),dim=1)
	end do

	!/* survival probabilities
	surv(:,ts:tt) = surv_1980_2055(js:jf,ts:tt)
	do i = tt+1,tf
		surv(:,i) = surv_1980_2055(js:jf,tt)
	end do

	!/* output results */
!	open(10,file='pop_0_100_dynamics.out')
!	open(11,file='pop_20_100_dynamics.out')
!	open(12,file='pop_retired.out')
!	open(13,file='pop_0_19.out')
!	open(14,file='pop_20_65.out')
!	open(15,file='pop_66_100.out')
!	open(16,file='pop_20_35.out')
!	open(17,file='pop_36_50.out')
!	open(18,file='pop_51_65.out')
!	open(20,file='pop_year_1980.out')
!	open(21,file='pop_year_1990.out')
!	open(22,file='pop_year_2000.out')
!	open(23,file='pop_year_2010.out')
!	open(24,file='pop_year_2020.out')
!		do i = ts,tf
!			write(10,*) pop_0_100(i)
!			write(11,*) pop_20_100(i)
!			write(12,*) frac_ret(i)
!			write(13,*) pop_0_19(i)
!			write(14,*) pop_20_65(i)
!			write(15,*) pop_66_100(i)
!			write(16,*) pop_20_35(i)
!			write(17,*) pop_36_50(i)
!			write(18,*) pop_51_65(i)
!		end do
!		do i = 0,jf
!			write(20,*) mu_tran(i,1980)
!			write(21,*) mu_tran(i,1990)
!			write(22,*) mu_tran(i,2000)
!			write(23,*) mu_tran(i,2010)
!			write(24,*) mu_tran(i,2020)
!		end do
!	close(10)
!	close(11)
!	close(12)
!	close(13)
!	close(14)
!	close(15)
!	close(16)
!	close(17)
!	close(18)
!	close(20)
!	close(21)
!	close(22)
!	close(23)
!	close(24)

	deallocate(surv_all)
	deallocate(surv_1980_2055)
	deallocate(mu_tran)

end subroutine demographics